module 'data.module' {
   constructor: '/'/2.

   /** check(X).
    *  Succeeds if <code>X</code> is instantiated to a module.
    */

   check(Module) :-
      data.term:check(Module),
      Module = '$module'(_, _, _).

   id(Module, Id) :-
      data.term:check(Module),
      Module = '$module'(Id, _, _).

   unify(M0, M1) :-
      data.term:check(M0),
      data.term:check(M1),
      M0 = '$module'(Id0, Closures0, Terms0),
      M1 = '$module'(Id1, Closures1, Terms1),
      (
         Id0 = Id1
      ->
         true
      ;
         data.list:append(Closures1, Closures0, Closures),
         data.list:append(Terms1, Terms0, Terms),
         data.list:iter(Terms) {
            do(Term) :-
               data.term:setarg1(1, Term, Id0),
               data.term:setarg1(2, Term, Closures),
               data.term:setarg1(3, Term, Terms).
         }
      ).

   /** predicate(Module, F, N).
    *  <prerequisite>
    *  <code>Module</code> is an instantiated module.
    *  </prerequisite>
    *  <prerequisite>
    *  <code>F</code> is an instantiated atom.
    *  </prerequisite>
    *  <prerequisite>
    *  <code>N</code> is an instantiated integer.
    *  </prerequisite>
    *  Succeeds if <code>Module</code> has a predicate with signature <code>F/N</code>.
    */

   predicate(Module, F, N) :-
      (
         'data.atom':check(F),
         'data.number':check(N)
      ->
         Module:'$has_predicate'('data.term':new(F, N))
      ;
         Module:'$has_predicate'(P),
         'data.term':functor(P, F, N)
      ).

   /** predicate(Module, F/N).
    *  Equivalent to <code>predicate(Module, F, N)</code>.
    */

   predicate(Module, F/N) :-
      'data.module':predicate(Module, F, N).
}